#include <Wire.h>
#include <OneWire.h>
#include <DallasTemperature.h> 
#include <LiquidCrystal.h>

#define ON_BUTTON A2
#define MINUS_BUTTON 2
#define PLUS_BUTTON 3
#define VENT A0
#define WATER_LEVEL A1
#define BUZZER 12
#define WATER_TEMP 13
#define RED_LIGHT 11
#define GREEN_LIGHT 10
#define BLUE_LIGHT 9
#define RS A4
#define E A3
#define D4 4
#define D5 5
#define D6 6
#define D7 7
#define LOW_LEVEL 50
#define MEDIUM_LEVEL 175
#define HIGH_LEVEL 300

int red_value, blue_value, green_value;
int is_on = 0, on_button, minus_button, plus_button;
float temp = 0, last_temp = 0, wanted_temp = 20;
int water_level = 0;

LiquidCrystal lcd(RS, E, D4, D5, D6, D7);
OneWire oneWire(WATER_TEMP);
DallasTemperature water_temp(&oneWire);

void reset_lcd() {
  lcd.setCursor(0,0);
  lcd.print("                ");
  lcd.setCursor(0,1);
  lcd.print("                ");
}
void print_start_lcd() {
  lcd.setCursor(0, 0);
  lcd.print("STARTING NOW..");
  lcd.setCursor(0,1);
  lcd.print("PLEASE WAIT..");
}

void RGB_color(int red_value, int green_value, int blue_value)
 {
  analogWrite(RED_LIGHT, red_value);
  analogWrite(GREEN_LIGHT, green_value);
  analogWrite(BLUE_LIGHT, blue_value);
}

void print_water_level() {
  lcd.setCursor(0, 1);
  lcd.print("LEVEL:");
  water_level = analogRead(WATER_LEVEL);
  if (water_level <= LOW_LEVEL) {
    lcd.print("EMPTY    ");
  } else if (water_level > LOW_LEVEL && water_level <= MEDIUM_LEVEL) {
    lcd.print("LOW   ");
  } else if (water_level > MEDIUM_LEVEL && water_level <= HIGH_LEVEL) {
    lcd.print("MEDIUM  ");
  } else if (water_level > HIGH_LEVEL) {
    lcd.print("HIGH  ");
  }
}

void print_temp() {
  water_temp.requestTemperatures();
  last_temp = temp;
  temp = water_temp.getTempCByIndex(0);
  if (last_temp == 0) {
    last_temp = temp;
  }
  temp = (last_temp + temp) / 2;
  if (temp >= wanted_temp - 0.3 && temp <= wanted_temp + 0.3) {
    analogWrite(VENT, 0);
    lcd.setCursor(0, 0);
    lcd.print("LIQUID IS AT    ");
    lcd.setCursor(0, 1);
    lcd.print("EXPECTED TEMP!  ");
    RGB_color(255, 0, 255);
    noTone(BUZZER);
  } else if (temp != -127) {
    RGB_color(125, 200, 0);
    tone(BUZZER, 1000);
    analogWrite(VENT, 255);
    lcd.setCursor(0, 0);
    lcd.print("C:");
    lcd.print(temp);
    lcd.print("  W:");
    lcd.print(wanted_temp);
    print_water_level();
    noTone(BUZZER);
  }
}

void lower_temp() {
  static unsigned long last_interrupt_time = 0;
  unsigned long interrupt_time = millis();
  if (interrupt_time - last_interrupt_time > 500) {
    wanted_temp -= 0.5;
   }
  last_interrupt_time = interrupt_time;
}

void higher_temp() {
  static unsigned long last_interrupt_time = 0;
  unsigned long interrupt_time = millis();
  if (interrupt_time - last_interrupt_time > 500) {
    wanted_temp += 0.5;
   }
  last_interrupt_time = interrupt_time;
}

void setup() {
  pinMode(RED_LIGHT, OUTPUT);
  pinMode(GREEN_LIGHT, OUTPUT);
  pinMode(BLUE_LIGHT, OUTPUT);
  pinMode(ON_BUTTON, INPUT);
  pinMode(MINUS_BUTTON, INPUT_PULLUP);
  interrupts();
  attachInterrupt(digitalPinToInterrupt(MINUS_BUTTON), lower_temp, FALLING);
  pinMode(PLUS_BUTTON, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(PLUS_BUTTON), higher_temp, FALLING);
  pinMode(BUZZER, OUTPUT);
  pinMode(VENT, OUTPUT);
  pinMode(WATER_LEVEL, OUTPUT);
  water_temp.begin();

  digitalWrite(ON_BUTTON, HIGH);
  digitalWrite(MINUS_BUTTON, HIGH);
  digitalWrite(PLUS_BUTTON, HIGH);

  lcd.begin(16, 2);
  lcd.print("NOT MEASURING..");
  lcd.setCursor(0,1);
  lcd.print("PRESS ON BUTTON");
  RGB_color(0, 255, 255);
}

void loop() {
  on_button = digitalRead(ON_BUTTON);
  minus_button = digitalRead(MINUS_BUTTON);
  plus_button = digitalRead(PLUS_BUTTON);
  if (on_button == LOW) {
    RGB_color(0, 0, 255);
    is_on = 1;
    reset_lcd();
    print_start_lcd();
    delay(2000);
    reset_lcd();
  }
  if (is_on) {
    print_temp();
  }
}
